#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef enum{OSOBA,KG} TUnit;
typedef enum{ERROR=-1,QUIT,OPEN,ADD,LIST,FIND,EXPORT} TAction;
typedef struct
{
   char producer[31];
   char regPlate[8];
   char type[11];
   enum {CAR,TRUCK} kind;
   union
   {
       struct
       {
           unsigned short maxPlaces;
           TUnit unit;
       } car;
       struct
       {
           unsigned maxLoad;
           TUnit unit;
       } truck;

   } capacity;
}TCar;

TAction menu();
int checkFile(FILE*);
TCar getCar();
void save(FILE*,TCar);
void list(FILE*);
TCar *find(FILE*, char*);
void listCar(TCar*);
void exportCars(FILE*,FILE*);
void readString(char *,int);

int main()
{
    TAction action;
    FILE *data = NULL,*out=NULL;
    TCar car;
    char expName[256];
    const char *FILENAME="data.dat";
    while((action=menu())!=QUIT)
    {
        switch(action)
        {
            case OPEN:
                data = fopen(FILENAME,"r+b");
                if(!data)
                {
                    data = fopen(FILENAME,"w+b");
                }
                break;
            case ADD:
                if(!checkFile(data))
                {
                    break;
                }
                car = getCar();

                save(data,car);
                fflush(data);
                break;
            case LIST:
                if(!checkFile(data))
                {
                    break;
                }
                list(data);
                break;
            case FIND:
                if(!checkFile(data))
                {
                    break;
                }
                char regPlate[8];
                //nacteni regPlate
                TCar *value = find(data,regPlate);
                if(!value)
                {
                    printf("Auto s registracni znackou %s nenalezeno.\n",regPlate);
                }
                else
                {
                    listCar(value);
                    free(value);
                }
                break;
            case EXPORT:
                printf("Soubor pro export: ");
                readString(expName,255);
                out = fopen(expName,"wt");
                exportCars(data,out);
                fclose(out);
                break;
            case QUIT:
                if(checkFile(data))
                {
                    fclose(data);
                }
                break;
            default:
                fprintf(stderr,"Zadana neplatna akce.\n");
        }
    }
    return 0;
}

TAction menu()
{
    int action;
    do{
        printf("1. Konec prace\n");
        printf("2. Otevrit kartoteku\n");
        printf("3. Pridat auto\n");
        printf("4. Vypsat kartoteku\n");
        printf("5. Hledat auto\n");
        printf("6. Export do CSV\n");
        scanf("%d",&action);
    }while(action<1 || action >6);
    getchar();
    return action-1;
}

int checkFile(FILE* file)
{
    if(!file)
    {
        fprintf(stderr,"1. krok musi byt otevreni kartoteky.\n");
        return 0;
    }
    return 1;
}

TCar getCar()
{
    TCar car;
    printf("Vyrobce: ");
    readString(car.producer,30);
    printf("Registracni znacka: ");
    readString(car.regPlate,7);
    printf("Typ: ");
    readString(car.type,10);
    printf("Druh (1 - osobni,2 - nakladni):");
    int kind;
    do
    {
        scanf("%d",&kind);
    }while(kind<1 || kind >2);
    car.kind = kind - 1;
    if(car.kind == CAR)
    {
        printf("Max pocet mist:");
        scanf("%hu",&(car.capacity.car.maxPlaces));
        car.capacity.car.unit = OSOBA;
    }
    else
    {
        printf("Maximalni nosnost (KG):");
        scanf("%u",&(car.capacity.truck.maxLoad));
        car.capacity.truck.unit = KG;
    }
    return car;
}

void readString(char *value,int maxSize)
{
    fgets(value,maxSize,stdin);
    char *nl = strchr(value,'\n');
    if(nl)
    {
        *nl = 0;
    }
    else
    {
        while(getchar()!='\n');
    }
}

void save(FILE* file,TCar car)
{
    fseek(file,0,SEEK_END);
    fwrite(&car,sizeof(TCar),1,file);
    fflush(file);
}

void list(FILE* file)
{
    fseek(file,0,SEEK_SET);
    TCar car;
    do
    {
        fread(&car,sizeof(TCar),1,file);
        if(!feof(file))
        {
            listCar(&car);
        }
    }while(!feof(file));
}

TCar *find(FILE* file, char* plate)
{
    TCar *car = malloc(sizeof(TCar));
    fseek(file,0,SEEK_SET);
    fread(car,sizeof(TCar),1,file);
    while(!feof(file))
    {
        if(strcmp(car->regPlate,plate)==0)
        {
            return car;
        }
        fread(car,sizeof(TCar),1,file);
    }
    free(car);
    return NULL;
}

void listCar(TCar* car)
{
    printf("==Auto s registraci: %s\n",car->regPlate);
    printf("Znacka: %s\n",car->producer);
    printf("Typ: %s\n",car->type);
    printf("Druh: %s\n",(car->kind==TRUCK?"nakladni":"osobni"));
    if(car->kind==CAR)
    {
        printf("\tmist: %d osob\n",car->capacity.car.maxPlaces);
    }
    else
    {
        printf("\tnosnost: %d kg\n",car->capacity.truck.maxLoad);
    }
    printf("==\n\n");

}

void exportCars(FILE* input,FILE* output)
{
    fseek(input,0,SEEK_SET);
    fseek(output,0,SEEK_SET);

    do
    {
        TCar car;
        fread(&car,sizeof(TCar),1,input);
        if(!feof(input))
        {
            fprintf(output,"%s,%s,%d,",car.producer,car.type,car.kind);
            if(car.kind==0)
            {
                fprintf(output,"%hu,%d",car.capacity.car.maxPlaces,car.capacity.car.unit);
            }
            else
            {
                fprintf(output,"%u,%d",car.capacity.truck.maxLoad,car.capacity.truck.unit);
            }
            fprintf(output,"\n");
        }
    }while(!feof(input));
    fflush(output);
}

